<?php

namespace Teknisa\CADFIN\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Sdoccorr;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\CADFIN\Factories\FluxoDeCaixaFactory as FluxoDeCaixaFactory;

/**
 * Class FluxoDeCaixaService
 * @package Teknisa\CADFIN\Services
 */
class FluxoDeCaixaService {
    protected $entityManager;
    protected $fluxoDeCaixaFactory;

    /**
     * @param EntityManager $entityManager
     * @param FluxoDeCaixaFactory $fluxoDeCaixaFactory
     */
    public function __construct(EntityManager $entityManager, FluxoDeCaixaFactory $fluxoDeCaixaFactory)
    {
        $this->entityManager = $entityManager;
        $this->fluxoDeCaixaFactory = $fluxoDeCaixaFactory;
    }

    /**
     * Atualiza Fluxo de Caixa
     * @param $nrorg
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $vrmovfluxo
     * @param $idmovfluxo
     * @param $idoperacao
     * @return \Financeiro\Entities\Movfluxo|null
     * @throws \Exception
     */
    public function atualizaFluxoCaixa(
        $nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $vrmovfluxo, $idmovfluxo, $idoperacao
    ){
        try {

            // Verifica Identificador Movimentação de Fluxo
            if (! in_array($idmovfluxo,['R','E','P'])){
                throw new \Exception('Identificador de Movimento de Fluxo inválido.');
            }
            $dtmovfluxo = self::formataData($dtmovfluxo);

            // Busca Movimentação de Fluxo
            $movfluxo = $this->fluxoDeCaixaFactory->buscaMovFluxo(
                $nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo
            );

            // Verifica Existência Movimentação de Fluxo
            if ($movfluxo) {
                // Atualiza Movimentação de Fluxo
                $movfluxo = $this->fluxoDeCaixaFactory->atualizaMovFluxo(
                    $movfluxo, $vrmovfluxo, $idoperacao
                );
            } else {
                // Insere Movimentação de Fluxo
                $movfluxo = $this->fluxoDeCaixaFactory->insereMovFluxo(
                    $nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo, $vrmovfluxo, $idoperacao
                );
            }

            return $movfluxo;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Atualiza Saldo Conta Corrente
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $nrorg
     * @param $valor
     * @throws \Exception
     */
    public function atualizaSaldoContCorr($cdreducont, $dtmovbanc, $nrorg, $valor){
        try {

            // Busca Saldo Conta Corrente
            $saldos = $this->fluxoDeCaixaFactory->buscaSaldoContCorr($cdreducont, $dtmovbanc, $nrorg);

            // Busca Saldo Anterior
            $saldoInicial = $this->fluxoDeCaixaFactory->getSaldoAnterior($nrorg, $cdreducont, $dtmovbanc->format('d/m/Y'));

            // Verifica Existência de Saldo(s)
            if (!$saldos) {
                // Insere Saldo Conta Corrente
                $sdoccorr = $this->fluxoDeCaixaFactory->insereSaldoContaCorrente($nrorg, $cdreducont, $dtmovbanc, round($saldoInicial + $valor, 2));

                $this->entityManager->persist($sdoccorr);

            } else {

                /** @var Sdoccorr $saldo */
                $saldo = $saldos[0];

                // Verifica Existência Saldo para Data Passada por Parámetro, caso não exista Inclui Saldo
                if ($saldo->getDtsdoccorr()->format('d/m/Y') != $dtmovbanc->format('d/m/Y')) {
                    //Insere o saldo para a data de movimento
                    $sdoccorr = $this->fluxoDeCaixaFactory->insereSaldoContaCorrente($nrorg, $cdreducont, $dtmovbanc, round($saldoInicial + $valor, 2));
                    $this->entityManager->persist($sdoccorr);
                }

                $ultimosaldo = 0;
                $existeSaldoImplantado = false;

                // Percorre Saldo(s) Conta Corrente
                /** @var Sdoccorr $saldo */
                foreach ($saldos as $saldo) {
                    // Verifica Existência Valor Saldo Implantado
                    if ($saldo->getVrsdoimpla()) {
                        $saldo->setVrsdoccorr($saldo->getVrsdoimpla());
                        $ultimosaldo = $saldo->getVrsdoimpla();

                        $existeSaldoImplantado = true;
                    } else {
                        // Verifica Existência Saldo Implantado
                        if ($existeSaldoImplantado) {
                            $saldo->setVrsdoccorr(round($ultimosaldo + $valor, 2));
                            $ultimosaldo = round($ultimosaldo + $valor, 2);
                        } else {
                            $saldo->setVrsdoccorr(round($saldo->getVrsdoccorr() + $valor, 2));
                        }
                    }

                    // Atualiza Saldo Conta Corrente
                    $this->entityManager->persist($saldo);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaOrcamento($cdempresa, $cdfilial, $cdclassfina, $cdfiliproc, $dataLancamento, $valor){
        try {
            $paramgeral = $this->fluxoDeCaixaFactory->getParamgeral();
            if ($paramgeral){
                if ($paramgeral->getIdconsorcfin() == 'S'){
                    $primeiroDiaMes = mktime(0, 0, 0, date('m', $dataLancamento) , 1 , date('Y', $dataLancamento));
                    $ultimoDiaMes = mktime(23, 59, 59, date('m', $dataLancamento), date("t", $dataLancamento), date('Y', $dataLancamento));
                    $filialOrcamento = ($paramgeral->getIdtporcafina() == 'F') ? $cdfilial : null;
                    $orcafina = $this->fluxoDeCaixaFactory->getOrcamentoFinanceiro($nrorg, $cdempresa, $filialOrcamento, $cdclassfina, $primeiroDiaMes, $ultimoDiaMes);
                    if ($orcafina){
                        if ($valor < 0){
                            $orcafina->setVrrealorca($orcafina->getVrrealorca() + $valor);
                            $this->entityManager->persist($orcafina);
                        }else{
                            if (($orcafina->getVrrealorca() + $valor) <= ($orcafina->getVrorcafina() + $orcafina->getVrextoleorca())){
                                $orcafina->setVrrealorca($orcafina->getVrrealorca() + $valor);
                                $this->entityManager->persist($orcafina);
                                $this->entityManager->flush();
                            }else{
                                throw new \Exception("Operação bloqueada! Conta não pode ser incluída por ter superado valor orçado.");
                            }
                        }
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|DateTime|string
     */
    public static function formataData($data)
    {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

}
